function  compHeatSchemes
% compHeatSchemes  Compare FTCS, BTCS, and Crank-Nicolson schemes
%                  for solving the 1D heat equation

% --- Set constants to be consistent with BTCS
alfa = 0.1;  L = 1;  tmax=2;
rsafe = 0.49999;  % FTCS is satable when r<0.5

% --- Demonstrate effect of reducing x-direction mesh spacing
% --- Specify nx and compute nt consistent with stability limit
nx = [8 16 32 64 128 256];                           dx = L./(nx-1);
nt = ceil( 1 + alfa*tmax*((nx-1).^2)/(rsafe*L^2) );  dt = tmax./(nt-1);
errB = [];   errF = [];   errCN = [];

fprintf('\nReduce both dx and dt within the FTCS stability limit\n\n');
fprintf('                 -------------  Errors  ------------\n');
fprintf('    nx     nt        FTCS       BTCS         CN\n');

errF = zeros(length(dx),1);  errB = errF;  errCN = errF;  %  preallocate
for i=1:length(nx)
  r = alfa*dt(i)/dx(i)^2;
  if r<0.5
    errF(i) = demoFTCS(nx(i),nt(i));
  else
    errF(i) = NaN;
  end
  errB(i)  = demoBTCS(nx(i),nt(i));
  errCN(i) = demoCN(nx(i),nt(i));
  fprintf(' %5d  %5d  %11.3e  %11.3e  %11.3e\n',nx(i),nt(i),errF(i),errB(i),errCN(i));
end

imid = fix(length(errB)/2);
eref = errB(imid)*(dx/dx(imid)).^2;
loglog(dx,errF,'o',dx,errB,'*',dx,errCN,'s',dx,eref,'k--');
legend('FTCS','BTCS','CN','E \propto\Delta x^2','Location','northwest')
xlabel('\Delta x');   ylabel('E(\Delta x,\Delta t)');

% --- Demonstrate effect of reducing time step
nx = 256;                       dx = L/(nx-1);
nt = [16 32 64 128 512 1024];   dt = tmax./nt;

fprintf('\nReduce dt while holding dx = %f',dx);
fprintf(' (L=%3.1f, nx=%d) constant\n\n',L,nx);
fprintf('                 -------------  Errors  ------------\n');
fprintf('    nx     nt        FTCS       BTCS         CN\n');

errF = zeros(length(dt),1);  errB = errF;  errCN = errF;  %  preallocate
for i=1:length(dt)
  r = alfa*dt(i)/dx^2;
  if r<0.5
    errF(i) = demoFTCS(nx,nt(i));
  else
    errF(i) = NaN;
  end
  errB(i)  = demoBTCS(nx,nt(i));
  errCN(i) = demoCN(nx,nt(i));
  fprintf(' %5d  %5d  %11.3e  %11.3e  %11.3e\n',nx,nt(i),errF(i),errB(i),errCN(i));
end

imid = fix(length(errF)/2) - 1;
eref   = errB(imid)*dt/dt(imid);
eCNref = errCN(imid)*(dt/dt(imid)).^2;
figure
loglog(dt,errF,'o',dt,errB,'*',dt,errCN,'s',dt,eref,'k--',dt,eCNref,'k:');
legend('FTCS','BTCS','CN','E\propto\Delta t','E\propto\Delta t^2','Location','northwest')
xlabel('\Delta t');   ylabel('E(\Delta x,\Delta t)');
text(2e-3,0.3,sprintf('\\Delta x =%9.1e (constant)',dx),'FontSize',14,'FontWeight','demi')

end