function errout = demoBTCS(nx,nt)
% demoBTCS  Solve 1D heat equation with BTCS scheme on a uniform mesh
%
% Synopsis:  demoBTCS
%            demoBTCS(nx)
%            demoBTCS(nx,nt)
%            err = demoBTCS(...)
%
% Input:  nx = number of x-direction nodes  Default:  nx = 11
%         nt = number of time steps         Default:  nt = 50
%
% Output:  err = (optional) error in the numerical solution
if nargin<1 || isempty(nx),  nx = 11;  end
if nargin<2 || isempty(nt),  nt = 50;  end

% --- Assign physical and mesh parameters
alfa = 0.1;  L = 1;  tmax = 2;  % Diffusion coefficient, domain length and max time
dx = L/(nx-1);       dt = tmax/(nt-1);

% --- Coefficients of the tridiagonal system
b = (-alfa/dx^2)*ones(nx,1);     %  Super diagonal: coefficients of u(i+1)
c = b;                           %  Subdiagonal:    coefficients of u(i-1)
a = (1/dt)*ones(nx,1) - (b+c);   %  Main Diagonal:  coefficients of u(i)
a(1) = 1;     b(1) = 0;          %  Fix coefficients of boundary nodes   
a(end) = 1;   c(end) = 0;
[e,f] = tridiagLU(a,b,c);        %  Save LU factorization

% --- Assign IC and save BC values in ub.  IC creates u vector
x = linspace(0,L,nx)';   u = sin(pi*x/L);   ub = [0 0];

% --- Loop over time steps
for k=2:nt
  d = [ub(1); u(2:nx-1)/dt;  ub(2)];   %  Update RHS, preserve BC
  u = tridiagLUsolve(e,f,b,d);         %  Solve the system
end

% --- Return error, or plot exact and finite difference solutions at last step
ue = sin(pi*x/L)*exp(-tmax*alfa*(pi/L)^2);  err = norm(u-ue)/sqrt(nx);
if nargout>0
  errout=err;  return;
else
  fprintf('Error in BTCS solution = %f\n',err)
  xeplot = linspace(0,L);  %  Many points gives smooth exact curve
  ueplot = sin(pi*xeplot/L)*exp(-tmax*alfa*(pi/L)^2);
  plot(x,u,'o--',xeplot,ueplot,'-');
  xlabel('x');  ylabel('u','Rotation',0);   legend('BTCS','Exact');
end
end