function errout = demoCN(nx,nt)
% demoCN  Solve 1D heat equation with Crank-Nicolson scheme on a uniform mesh
%
% Synopsis:  demoCN
%            demoCN(nx)
%            demoCN(nx,nt)
%            err = demoCN(...)
%
% Input:  nx = number of x-direction nodes  Default:  nx = 11
%         nt = number of time steps         Default:  nt = 50
%
% Output:  err = (optional) error in the numerical solution
if nargin<1 || isempty(nx),  nx = 11;  end
if nargin<2 || isempty(nt),  nt = 50;  end

% --- Assign physical and mesh parameters
alfa = 0.1;  L = 1;  tmax = 2;  % Diffusion coefficient, domain length and max time
dx = L/(nx-1);       dt = tmax/(nt-1);

% --- Coefficients of the tridiagonal system
b = (-alfa/2/dx^2)*ones(nx,1);   %  Super diagonal: coefficients of u(i+1)
c = b;                           %  Subdiagonal:    coefficients of u(i-1)
a = (1/dt)*ones(nx,1) - (b+c);   %  Main Diagonal:  coefficients of u(i)
at = (1/dt + b + c);             %  Coefficient of u_i^k on RHS
a(1) = 1;     b(1) = 0;          %  Fix coefficients of boundary nodes   
a(end) = 1;   c(end) = 0;
[e,f] = tridiagLU(a,b,c);        %  Save LU factorization

% --- Assign IC and save BC values in ub.  IC creates u vector
x = linspace(0,L,nx)';   u = sin(pi*x/L);   ub = [0 0];

% --- Loop over time steps
for k=2:nt
  % --- Update RHS for all equations, including those on boundary
  d =  - [0;       c(2:end-1).*u(1:end-2); 0] ...
       + [ub(1);  at(2:end-1).*u(2:end-1); ub(2)] ...
       - [0;       b(2:end-1).*u(3:end);   0];
  u = tridiagLUsolve(e,f,b,d);                   %  Solve the system
end

% --- Return error, or plot exact and finite difference solutions at last step
ue = sin(pi*x/L)*exp(-tmax*alfa*(pi/L)^2);  err = norm(u-ue)/sqrt(nx);
if nargout>0
  errout=err;  return;
else
  fprintf('Error in CN solution = %f\n',err)
  xeplot = linspace(0,L);  %  Many points gives smooth exact curve
  ueplot = sin(pi*xeplot/L)*exp(-tmax*alfa*(pi/L)^2);
  plot(x,u,'o--',xeplot,ueplot,'-');
  xlabel('x');  ylabel('u','Rotation',0);   legend('CN','Exact');
end

end