function demoCNBC(nx,nt,L,tmax)
% demoCNBC  BC Test for Crank-Nicolson solutions to heat equation.
%           Solution to model problem is decay of a half sine wave 
%
% Synopsis:  demoCNBC
%            demoCNBC(nx)
%            demoCNBC(nx,nt)
%            demoCNBC(nx,nt,L)
%
% Input:  nx = number of x-direction nodes.  Default:  nx = 11
%         nt = number of time steps.         Default:  nt = 50
%         L = length of the domain.          Default:  L = 1
%         tmax = max time for the solution.  Default:  tmax = 2

if nargin<1 || isempty(nx);   nx = 11;   end
if nargin<2 || isempty(nt);   nt = 50;   end
if nargin<3 || isempty(L);    L = 1;     end
if nargin<4 || isempty(tmax); tmax = 2;  end

% --- Solve heat equation on 0 <= x <= L with Dirichlet BCs
[U,x,~] = heatCNBC(nx,nt,[],[],L,tmax);
ue = sin(pi*x/L)*exp(-0.1*tmax*pi^2/L^2);
err = norm(U(:,end) - ue)/sqrt(length(x));
fprintf('Error in CN solution for whole domain      = %e\n',err)
plot(x,U(:,1),'b.',x,U(:,end),'o',x,ue,'r--');
axis([0 L 0 1.5]);  xlabel('x');  ylabel('u');  legend('IC','CN','Exact');

% --- Solve heat equation on 0 <= x <= L/2 with Dirichlet BC at x=0
%     and symmetry BC at x=L/2.  Initial condition must be specified
%     explicitly because default IC in heatCNBC is a half sine wave.
L2 = L/2;
nx2 = 1 + floor(nx/2);     %  Halve the nodes to compare with full domain solution 
x2 = linspace(0,L/2,nx2)'; %  Uniform mesh on half domain
ubc = [1 0 0; 2 0 0];      %  Matrix to specify BC for heatCNBC
u0 = sin(pi*x2/L);         %  Initial condition on half domain
ue2 = sin(pi*x2/L)*exp(-0.1*tmax*pi^2/L^2);  %  Exact solution on half domain
[U2,x2,~] = heatCNBC(nx2,nt,ubc,u0,L2);
err = norm(U2(:,end) - ue2)/sqrt(length(x2));
fprintf('Error in CN solution for left half domain  = %e\n',err)
figure;  subplot(1,2,1);  plot(x2,u0,'b-',x2,U2(:,end),'o',x2,ue2,'r--')
axis([0 L2 0 1.5]);  xlabel('x');   ylabel('u');
legend('IC','CN','Exact','Location','northwest');

% --- Solve heat equation on L/2 <= x <= L with symmetry BC at x=0
%     and Dirichlet BC at x=L/2.  Initial condition and exact solution
%     must be evaluated at shifted x coordinate:  xhat = x + L/2
ubc = [2 0 0; 1 0 0];      %  Matrix to specify BC for heatCNBC
u0 = sin(pi*(x2+L2)/L);    %  Initial condition on shifted half domain
ue3 = sin(pi*(x2+L2)/L)*exp(-0.2*pi^2/L^2);  %  Exact solution on shifted half domain
[U3,x3,~] = heatCNBC(nx2,nt,ubc,u0,L2);
err = norm(U3(:,end) - ue3)/sqrt(length(x3));
fprintf('Error in CN solution for right half domain = %e\n',err)
subplot(1,2,2);  plot(x3+L2,u0,'b-',x3+L2,U3(:,end),'o',x3+L2,ue3,'r--')
axis([L2 L 0 1.5]);  xlabel('x');  legend('IC','CN','Exact');

end