function errout = demoFTCS(nx,nt)
% demoFTCS  Solve 1D heat equation with FTCS scheme on a uniform mesh
%
% Synopsis:  demoFTCS
%            demoFTCS(nx)
%            demoFTCS(nx,nt)
%            err = demoFTCS(...)
%
% Input:  nx = number of x-direction nodes  Default:  nx = 11
%         nt = number of time steps         Default:  nt = 50
%
% Output:  err = (optional) error in the numerical solution
if nargin<1 || isempty(nx),  nx = 11;  end
if nargin<2 || isempty(nt),  nt = 50;  end

% --- Assign physical and mesh parameters
alfa = 0.1;  L = 1;  tmax = 2;  % Diffusion coefficient, domain length and max time
dx = L/(nx-1);       dt = tmax/(nt-1);
r = alfa*dt/dx^2;    r2 = 1 - 2*r;

% --- Assign IC and BC.  u is initialized to a vector that includes BC
x = linspace(0,L,nx)';    u = sin(pi*x/L);

% --- Loop over time steps
for k=2:nt
  uold = u;    % prepare for next step
  for i=2:nx-1
    u(i) = r*uold(i-1) + r2*uold(i) + r*uold(i+1);
  end
end

% --- Return error, or plot exact and finite difference solutions at last step
ue = sin(pi*x/L)*exp(-tmax*alfa*(pi/L)^2);  err = norm(u-ue)/sqrt(nx);
if nargout>0
  errout=err;  return;
else
  fprintf('Error in FTCS solution = %f\n',err)
  xeplot = linspace(0,L);  %  Many points gives smooth exact curve
  ueplot = sin(pi*xeplot/L)*exp(-tmax*alfa*(pi/L)^2);
  plot(x,u,'o--',xeplot,ueplot,'-');
  xlabel('x');  ylabel('u','Rotation',0);   legend('FTCS','Exact');
end

end