function demoHotPot(tmax,Tp,Tair,htop,hbot,nx)
% demoHotPot  Use heatCNBC to model placing a hot pot on a table
%
% Synopsis:  demoHotPot
%            demoHotPot(tmax)
%            demoHotPot(tmax,Tp)
%            demoHotPot(tmax,Tp,Tair)
%            demoHotPot(tmax,Tp,Tair,htop)
%            demoHotPot(tmax,Tp,Tair,htop,hbot)
%            demoHotPot(tmax,Tp,Tair,htop,hbot,nx)
%
% Input: tmax = max time for the solution.  Default:  tmax = 240 seconds
%        Tp   = temperature of the pot.  Default: Tp = 85 (C)
%        Tair = temperature of air below the table. Default: Tair = 20 (C)
%        htop = thermal contact conductance between pot and table top.
%               Default:  htop = 100 (W/m^2/C)
%        hbot = convection coefficient between table bottom and air.
%               Default:  hbot = 5 (W/m^2/C)
%        nx = number of x-direction nodes   Default:  nx = 51

if nargin<1 || isempty(tmax),  tmax = 120;   end
if nargin<2 || isempty(htop),  Tp = 85;      end
if nargin<3 || isempty(hbot),  Tair = 20;    end
if nargin<4 || isempty(htop),  htop = 100;   end
if nargin<5 || isempty(hbot),  hbot = 5;     end
if nargin<6 || isempty(nx),    nx = 51;      end

% --- Define physical properties for table and boundary conditions
%     Thermophysical properties of oak from F.P. Incropera and D.P. Dewitt,
%     Introduction to Heat Transfer, 2nd ed., 1990, Wiley and Sons
rho = 545;       %  density of oak (kg/m^3)
k = 0.17;        %  thermal conductivity of oak, across the grain (W/m/C)
c = 2385;        %  specific heat capacity of oak (J/kg/K)
alfa = k/rho/c;  %  thermal diffusivity (m^2/s)
L = 2e-2;        %  Table thickness (m)

% --- Use relaxation time of table material to specify time step size
tau = L^2/alfa;     %  Relaxation time for the heat condution (s)
dt = tau/1000;      %  Estimate a good time step (s); dt calculated in heatCNBC
nt = ceil(tmax/dt); %  Number of time steps

% --- Specify initial and boundary conditions
u0 = Tair*ones(nx,1);
ubc = [2 (-htop*Tp/k) (htop/k);  2  (hbot*Tair/k)  (-hbot/k)];

% --- Solve the heat equation and plot the results
[U,x,t] = heatCNBC(nx,nt,ubc,u0,L,tmax,alfa);
plotHeat(U,100*x,t,floor(nt/5))
xlabel('x  (cm)');   ylabel('T  ({{}^\circ}C)');  ylim([Tair-5, Tp])

end