function errout = demoUpwind(nx,nt,plotWave)
% demoUpwind  Solve 1D advection equation with upwind scheme
%
% Synopsis:  demoUpwind
%            demoUpwind(nx)
%            demoUpwind(nx,nt)
%            demoUpwind(nx,nt,plotWave)
%            err = demoUpwind(...)
%
% Input:  nx = number of x-direction nodes     Default:  nx = 101
%         nt = number of time steps            Default:  nt = 101
%         plotWave = flag to turn on plotting. Default:  plotWave = true
%
% Output:  err = (optional) error in the numerical solution
if nargin<1 || isempty(nx),    nx = 101;    end
if nargin<2 || isempty(nt),    nt = 101;    end
if nargin<3 || isempty(plotWave), plotWave = true;  end

% --- Assign physical parameters, create mesh and mesh parameters
U = 2;  L = 10;  tmax = 0.9*L/U;  % Wave speed, domain length and max time
x = linspace(0,L,nx)';  dx = L/(nx-1);  dt = tmax/(nt-1);  c = U*dt/dx;

% --- Assign IC and BC.  IC creates u vector.
x0 = L/10;  s = 0.3;
ue = @(x,xc,s)(exp(-((x-xc)/s).^2));  % analytical solution for moving pulse
u = ue(x,x0,s);
if plotWave, plot(x,u,'b:','Linewidth',1.5);  axis([0 L -0.25 1.5]);  hold('on');  end

% --- Loop over time steps.  Compare results at t in tshow
tshow = [(L/4 -x0)/U  (L/2-x0)/U  (3*L/4-x0)/U ];
for k=2:nt
  t = (k-1)*dt;  uold = u;    % prepare for next step
  for i=2:nx
    u(i) = uold(i) - c*( uold(i) - uold(i-1) );
  end
  if plotWave && any( abs(tshow-t)<0.52*dt )
    plot(x,u,'r--.',x,ue(x,x0+U*t,s),'k-');
    text(x0+U*t+1.5*s,max(u),sprintf('t = %-4.2f',t));
  end
end
if plotWave, legend('IC','Upwind','Exact'); xlabel('x');  ylabel('u');  end

% --- Compute error at last step
err = norm(u-ue(x,x0+U*t,s))/sqrt(nx);
if nargout>0
  errout=err;  return;
else
  fprintf('Error in upwind solution = %f\n',err)
end

end