function errout = demoWaveCD(nx,nt,plotWave)
% demoWaveCD  Solve the 1D wave equation with the central difference scheme
%
% Synopsis:  demoWaveCD
%            demoWaveCD(nx)
%            demoWaveCD(nx,nt)
%            demoWaveCD(nx,nt,plotWave)
%            err = demoWaveCD(...)
%
% Input:  nx = number of x-direction nodes     Default:  nx = 101
%         nt = number of time steps            Default:  nt = 101
%         plotWave = flag to turn on plotting. Default:  plotWave = true
%
% Output:  err = (optional) average error in the numerical solution
if nargin<1 || isempty(nx),       nx = 101;         end
if nargin<2 || isempty(nt),       nt = 101;         end
if nargin<3 || isempty(plotWave), plotWave = true;  end

% --- Assign physical parameters and mesh parameters
a = 2;  L = 10;  tmax = L/a;         % Wave speed, domain length and max time
dx = L/(nx-1);   dt = tmax/(nt-1);   r = a*dt/dx;   r2 = r^2;  r3 = 2*(1-r2);
% --- Assign IC and BC.  IC creates the u vector
x = linspace(0,L,nx)';  u = sin(pi*x/L);  uold = u;  g0 = 0;  h0 = 0;
if plotWave  % Plot Initial Condition
  plot(x,u,'b-'); hold('on'); axis([0 L -1.25 1.25]);  xlabel('t'); ylabel('u')
end
% --- First time step uses ghost node formulation with initial conditions
for i=2:nx-1
  u(i) = 0.5*( ( r3 + h0*dt )*uold(i) + g0*dt + r2*( uold(i-1)+uold(i+1) ) );
end
% --- Loop over time steps.  Compare results when t is close to any elment in tshow
tshow = [tmax/4  tmax/3  tmax/2  3*tmax/4  tmax];
for k=3:nt
  t = (k-1)*dt;  uoldold = uold;   uold = u;  % prepare for next step
  for i=2:nx-1
    u(i) = r3*uold(i) + r2*( uold(i-1)+uold(i+1) ) - uoldold(i);
  end
  if plotWave && any( abs(tshow-t)<0.51*dt )
    plot(x,u,'ro','Markersize',2);  plot(x,cos(pi*a*t/L)*sin(pi*x/L),'k-')
    text(0.48*L,u(fix(nx/2))+0.08*sign(u(fix(nx/2))),sprintf('t = %-4.2f',t))
  end
end
err = norm(u-cos(pi*a*t/L)*sin(pi*x/L));  %  Average error at last step
if nargout>0
  errout=err;  return;
else
  fprintf('Error = %g at t = %g\n',err,t);
end

end