function [U,x,t] = heatCNBC(nx,nt,ubc,u0,L,tmax,alfa)
% demoCNBC  Crank-Nicolson scheme for 1D heat equation with mixed BC
%
% Synopsis:  [U,x,t] = heatCNBC
%            [U,x,t] = heatCNBC(nx)
%            [U,x,t] = heatCNBC(nx,nt)
%            [U,x,t] = heatCNBC(nx,nt,ubc)
%            [U,x,t] = heatCNBC(nx,nt,ubc,u0)
%            [U,x,t] = heatCNBC(nx,nt,ubc,u0,L)
%            [U,x,t] = heatCNBC(nx,nt,ubc,u0,L,tmax)
%            [U,x,t] = heatCNBC(nx,nt,ubc,u0,L,tmax,alfa)
%
% Input:  nx = number of x-direction nodes  Default:  nx = 11
%         nt = number of time steps         Default:  nt = 50
%         ubc = 2-by-3 matrix used to specify boundary conditions.
%               ubc(1,:) prescribes the left (x=0) boundary,  ubc(2,:)
%               prescribes the right (x=L) boundary.  ubc(:,1) indicates
%               the type of BC.  If ubc(1,1) = 1, then the BC is Dirichlet
%               and ubc(1,2) is the value of u at x=0.  If ubc(1,1) = 2,
%               then the BC at x=0 is is du/dx = g + h*u, where
%               g = ubc(1,2) and h = ubc(1,3). Default:  Dirichlet
%               conditions u=0 at both x=0 and x=L are imposed.
%         u0 = vector containing IC.  Default: u0 = sin(pi*x) where
%              x is a column vector created by linspace(0,1)'
%         L = (scalar) length of the domain.          Default: L = 1
%         tmax = (scalar) max time for the solution.  Default: tmax = 2
%         alfa = (scalar) diffusion coefficient.      Default: alfa = 0.1
%
%  Note:  If the null matrix, i.e. [], is used for an input parameter
%         the default value of that parameter is used.  Thus,
%         [U,x,t] = heatCNBC([],101,[],[],5) obtains the solution with
%         the default values of nx, ubc, and u0, but with nt=101 and L=5.
%
% Output: U = matrix containing solution:  U(:,k) is U(x) at t = t(k)
%         t = vector of times at which solution is obtained (time nodes)
if nargin<1 || isempty(nx),   nx = 11;    end
if nargin<2 || isempty(nt),   nt = 50;    end
if nargin<3 || isempty(ubc),  ubc = [1,0,0; 1,0,0];   end
if nargin<5 || isempty(L),    L = 1;      end
if nargin<6 || isempty(tmax), tmax = 2;   end
if nargin<7 || isempty(alfa), alfa = 0.1; end

% --- Define the mesh and provide default initial condition
x = linspace(0,L,nx)';  dx = L/(nx-1);  dt = tmax/(nt-1);
if nargin<4 || isempty(u0),   u0 = sin(pi*x/L);   end

% --- Preallocate output arrays and assign initial condition
U = zeros(nx,nt);   t = linspace(0,tmax,nt);   U(:,1) = u0;

% --- Coefficients of the tridiagonal system
b = (-alfa/2/dx^2)*ones(nx,1);   %  Super diagonal: coefficients of u(i+1)
c = b;                           %  Subdiagonal: coefficients of u(i-1)
a = (1/dt)*ones(nx,1) - (b+c);   %  Main Diagonal: coefficients of u(i)
c(1) = 0;
b(end) = 0;

% --- Adjust boundary coefficients
switch ubc(1,1)
  case 1
    a(1)=1;  b(1)=0;
  case 2
    a(1) = 1/dt + alfa*(1 + ubc(1,3)*dx)/dx^2;
    b(1) = -alfa/dx^2;
  otherwise
    error('ubc(1,1) must be equal to 1 or 2');
end
switch ubc(2,1)
  case 1
    a(end)=1;  c(end)=0;
  case 2
    a(end) = 1/dt + alfa*(1 - ubc(2,3)*dx)/dx^2;
    c(end) = -alfa/dx^2;
  otherwise
    error('ubc(2,1) must be equal to 1 or 2');
end
[e,f] = tridiagLU(a,b,c);        %  Save LU factorization

% --- Loop over time steps
at = (1/dt + b + c);   %  coefficient of u_i^k
for k=1:nt-1
  % --- Update RHS for all equations except those on boundary
  d =  - [0;  c(2:end-1).*U(1:end-2,k); 0] ...
       + [0; at(2:end-1).*U(2:end-1,k); 0] ...
       - [0;  b(2:end-1).*U(3:end,k);   0];
  % --- Overwrite values in d(1) and d(end) to enforce BCs
  switch ubc(1,1)
    case 1
      d(1) = ubc(1,2);
    case 2
      d(1) = (1/dt - alfa*(1 + ubc(1,3)*dx)/dx^2)*U(1,k) ...
           + alfa*U(2,k)/dx^2 - 2*alfa*ubc(1,2)/dx;
    otherwise
      error('ubc(1,1) must be equal to 1 or 2');
  end
  switch ubc(2,1)
    case 1
      d(end) = ubc(2,2);
    case 2
      d(end) = (1/dt - alfa*(1 - ubc(2,3)*dx)/dx^2)*U(end,k) ...
           + alfa*U(end-1,k)/dx^2 + 2*alfa*ubc(2,2)/dx;
    otherwise
      error('ubc(2,1) must be equal to 1 or 2');
  end
  % --- Solve the system
  U(:,k+1) = tridiagLUsolve(e,f,b,d);
end
end