function errout = movieFTCS(nx,nt,delay)
% movieFTCS  Animate FTCS solution to 1D heat equation
%
% Synopsis:  movieFTCS
%            movieFTCS(nx)
%            movieFTCS(nx,nt)
%            err = movieFTCS(...)
%
% Input:  nx = number of x-direction nodes  Default:  nx = 11
%         nt = number of time steps         Default:  nt = 50
%         delay = time to wait after solution is drawn on each time
%                 step.  Increase delay for faster computers or
%                 for smaller meshes. Default:  delay = 0.05
%
% Output:  err = (optional) error in the numerical solution
if nargin<1 || isempty(nx),    nx = 11;    end
if nargin<2 || isempty(nt),    nt = 50;    end
if nargin<3 || isempty(delay),  delay = 0.05;  end

% --- Assign physical parameters, create mesh and mesh parameters
alfa = 0.1;  L = 1;  tmax = 2;  % Diffusion coefficient, domain length and max time
x = linspace(0,L,nx)';
dx = L/(nx-1);          dt = tmax/(nt-1);
r = alfa*dt/dx^2;       r2 = 1 - 2*r;

% --- Assign IC and BC.  u is initialized to a vector that includes BC
u = sin(pi*x/L);

% --- Create initial plot
plot(x,u,'k-');                                  %  Initial condition
hold('on');  p = plot(x,u,'r--.');  hold('off'); %  data that is updated
legend('Initial Condition','FTCS solution')
axis([0 L -1 2]);

% --- Loop over time steps
for k=2:nt
  uold = u;    % prepare for next step
  for i=2:nx-1
    u(i) = r*uold(i-1) + r2*uold(i) + r*uold(i+1);
  end
  set(p,'ydata',u);     %  Change data in the plot
  pause(delay)
end

% --- Return error, or plot exact and finite diff solutions at last step
ue = sin(pi*x/L)*exp(-tmax*alfa*(pi/L)^2);  err = norm(u-ue)/sqrt(nx);
if nargout>0
  errout=err;  return;
else
  fprintf('Error in the FTCS solution = %f\n',err)
end

end