function plotHeat(U,x,t,nskip,legendLoc,lineSymb)
% plotHeat  Plot profiles of finite-difference solutions to the heat equation
%
% Synopsis:  plotHeat(U,x,t)
%            plotHeat(U,x,t,nskip)
%            plotHeat(U,x,t,nskip,legendLoc)
%
% Input:  U = matrix of U(x,t) values.  U(:,k) is U(x) at time t(k)
%             plotHeat plots x-direction profiles of U at times determined
%             by the value of nskip (below)
%         x = vector of spatial nodes on the finite-difference mesh
%         t = vector of times at which solution is available
%         nskip = number of times to skip in the sequence of profile plots OR
%                 a list of times the profiles of U are desired.
%                 If length(nskip)>1, nskip is interpretted as a list of
%                 times at which profile plots are desired.
%                 If nksip<0, the sequence of times is chosen logarithmically
%                 This is useful for problems with very large time duration
%                 but with fast phenomena at the start
%                 Default: nskip = 2, plot every other time step.
%         legendLoc = (optional) string defining position of the legend
%                     for the plot.  Default:  legendLoc = 'best'
%                     See 'doc legend' for valid options
%
% Output:  Plot in a figure window

if nargin<4 || isempty(nskip),     nskip=2; end              %  Default skip pattern
if nargin<5 || isempty(legendLoc), legendLoc = 'best';  end  %  Default legend location

% --- Define string matrix of line styles that can be reused.
%     nsymb is the total number of line style strings (plot symbols)
%     In plot loop the statement isymb = 1 + rem(j-1,nsymb) is an
%     index in the range 1 <= isymb <= nsymb
if nargin<6 || isempty(lineSymb)
   lineSymb = {'k-';'b-.';'k-o';'m-v';'b-s';'k*-';'m-d';'b-+';'k-<';'m-h'};
end
nsymb = size(lineSymb,1);

% -- Determine list of times to show U(x,t)
if length(nskip)>1
  %  User has supplied a list of times instead of specifying skips
  tplot = nskip;                 %  tplot is the list of times
  kplot = zeros(size(tplot));    %  kplot is the list of indices in t
  for i = 1:length(tplot)
    [~,k] = min( abs( t-tplot(i) ) );   %  k is the closest index
    kplot(i) = k;
  end
elseif nskip<0
  np = length(t)/abs(nskip);    %  number of time steps to plot
  kplot = floor( logspace(0,log10(length(t)),np) );
else
  kplot = 1:nskip:length(t);
end
% -- Heuristic to add curve for another time
if t(kplot(end)) < 0.9*t(end)  %  If last t selected by kplot is far from t(end)
  kplot = [kplot, length(t)];  %  add t(end) to the list of times to plot U(x,t)
else
  kplot(end) = length(t);      %  Otherwise, make sure last time is t(end)
end

% --- Plot U(x,t):  each time is a different curve with different symbol
legstr = cell(1,length(kplot));
j = 0;                          %  j is index for line styles
for k=kplot                     %  loop over times selected by kplot
  j = j+1;
  hold('on');
  isymb = 1 + rem(j-1,nsymb);        %  cyclic index for line styles.
  plot(x,U(:,k),lineSymb{isymb,:});
  legstr{j} = sprintf(' t = %-4.2f',t(k));  %  Build legend string
end
hold('off');   legend(legstr,'Location',legendLoc);
xlabel('x');   ylabel('T(x,t)');

end